//
//  TCGBError.h
//  Gamebase
//
//  Created by NHNENT on 2016. 6. 20..
//  Copyright © 2016년 NHN Entertainment Corp. All rights reserved.
//


#ifndef TCGBError_h
#define TCGBError_h

#import <Foundation/Foundation.h>

extern NSString* const TCGBDomainGamebase;
extern NSString* const TCGBErrorUserinfoTransactionId;

/** TCGBError class represents a result of some APIs or an occured error.
 */
@interface TCGBError : NSError

/**---------------------------------------------------------------------------------------
 * @name Initialization
 *  ---------------------------------------------------------------------------------------
 */

/**
 Creates TCGBError instance.
 
 @param code error code
*/
+ (TCGBError *)resultWithCode:(NSInteger)code;

/**
 Creates TCGBError instance.
 
 @param code error code.
 @param message error message.
 */
+ (TCGBError *)resultWithCode:(NSInteger)code message:(NSString *)message;

/**
 Creates TCGBError instance.
 
 @param domain error domain.
 @param code error code.
 @param userInfo a dictionary with userInfo.
 */
+ (TCGBError *)resultWithDomain:(NSString *)domain code:(NSInteger)code userInfo:(NSDictionary *)userInfo;

+ (NSError *)errorWithError:(NSError *)error transactionId:(NSString *)transactionId;

/**---------------------------------------------------------------------------------------
 * @name Properties
 *  ---------------------------------------------------------------------------------------
 */

/**
 Error message
 */
- (NSString *)message;

/**
 Pretty JSON string
 */
- (NSString *)prettyJsonString;

/**
 JSON string
 */
- (NSString *)jsonString;



@end

#endif /* TCGBError_h */
